@@ -146,5 +146,7 @@ int Write_Remote_To_File (struct Reg_Remote remote_struct, char *dest, int size)
 int iswebfilename(char *string);
 int isstdout(char *string);
 int isnetwork(char *string);
+int GetConfig(int http_socket);
+int SetConfig(char *source, int http_socket);
 
 #endif				// _WEBPAGES_H_
