@@ -1,12 +1,12 @@
 /**********************************************************
  *
- * Snare for Solaris version 2.5.7
+ * Snare for Solaris version 3.0.0
  *
  * Author: InterSect Alliance Pty Ltd
  *
- * Copyright 2001-2006 InterSect Alliance Pty Ltd
+ * Copyright 2001-2007 InterSect Alliance Pty Ltd
  *
- * Last Modified: 10/01/2007
+ * Last Modified: 5/12/2007
  *
  * Available under the terms of the GNU Public Licence.
  * - See www.gnu.org
@@ -77,6 +77,7 @@
  *                  Matthias Ziegler for the code, and for releasing the
  *                  copyright back to us, so we can keep some of our
  *                  users legal teams happy!
+ *       5/12/2007  
  *
  **********************************************************
  *
@@ -148,6 +149,7 @@ static int AuditDestination = AUDIT_TO_STDOUT;		// Send to STDOUT unless otherwi
 static int SyslogDestination = 13;			// user.notice
 
 char hostid[MAX_HOSTID] = "";
+int restart_allow = 1;
 int remote_allow = 0;
 int remote_webport = 6161;
 char remote_restrictip[16] = "";
@@ -195,6 +197,7 @@ union semun {
 extern int errno;
 int caught_pipe;                // variable that lets us know when a SIGPIPE has been received
 int caught_usr1;                // variable that lets us know when a SIGUSR1 has been received
+int restart_agent=0;            // variable that lets us know whether to restart the agent or not
 
 uint_t CheckTrailer(char inbyte, int reset);
 
@@ -213,6 +216,14 @@ void kill_signal(sig)
 #ifdef DEBUG
 	DebugMsg("Caught kill");
 #endif //DEBUG
+	if (restart_agent && restart_allow) {
+#ifdef DEBUG
+	DebugMsg("Restarting agent in 5 seconds");
+#endif //DEBUG
+		sleep(5);
+		system("/etc/init.d/snare restart");
+		restart_agent = 0;
+	}
 	return;
 }
 
@@ -261,7 +272,6 @@ int main(int argc, char *argv[])
 	int auditstatus;
 	long auditpolicy=0;
 	int pid=0,pid2=0,pid3=0,pid4=0,ppid=0;	// process ids
-	int webpid = 0;
 	pthread_t thread; 
 
 	// NODE that matches the objective search term.
@@ -611,11 +621,17 @@ int main(int argc, char *argv[])
 								}
 							}
 							//DebugMsg("Audit failed, falling back to parent thread");
-							//if(argc == 1) { audit_off(); }
-							//kill(pid2,SIGTERM); waitpid(pid2,NULL,NULL);
-							//kill(pid3,SIGTERM); waitpid(pid3,NULL,NULL);
-							//kill(pid4,SIGTERM); waitpid(pid4,NULL,NULL);
-							//kill(ppid,SIGPIPE);
+							if(argc == 1) { audit_off(); }
+							if (restart_allow) {
+								restart_agent = 1;
+								DebugMsg("Snare agent restarting.\n");
+							} else {
+								DebugMsg("Snare agent exiting.\n");
+							}
+							kill(pid2,SIGTERM); waitpid(pid2,NULL,NULL);
+							kill(pid3,SIGTERM); waitpid(pid3,NULL,NULL);
+							kill(pid4,SIGTERM); waitpid(pid4,NULL,NULL);
+							kill(ppid,SIGPIPE);
 							exit(0);
 							break;
 						}
@@ -733,6 +749,12 @@ int main(int argc, char *argv[])
 										if (error==EINVAL) {
 											// Auditing dissapeared from under us?
 											DebugMsg("SNARE reports that auditing has been turned off. Exiting.\n");
+											if (restart_allow) {
+												restart_agent = 1;
+												DebugMsg("Restarting.\n");
+											} else {
+												DebugMsg("Exiting.\n");
+											}
 											// Just in case.
 											if (argc == 1) { audit_off(); }
 											unlink("/var/run/snarecore.pid");
@@ -764,11 +786,9 @@ int main(int argc, char *argv[])
 											// kill(pid2,SIGTERM); waitpid(pid2,NULL,NULL);
 											// kill(ppid,SIGTERM); // waitpid(pid,NULL,NULL);
 											if(argc == 1) { audit_off(); }
-											DebugMsg("killing pid2...");
+											restart_agent = 1;
 											kill(pid2,SIGTERM); waitpid(pid2,NULL,NULL);
-											DebugMsg("killing pid3...");
 											kill(pid3,SIGTERM); waitpid(pid3,NULL,NULL);
-											DebugMsg("killing pid4...");
 											kill(pid4,SIGTERM); waitpid(pid4,NULL,NULL);
 											kill(ppid,SIGPIPE);
 											if (remote_allow) {
@@ -782,6 +802,7 @@ int main(int argc, char *argv[])
 
 											// kill(ppid,SIGTERM); // waitpid(pid,NULL,NULL);
 											if(argc == 1) { audit_off(); }
+											restart_agent = 1;
 											kill(pid2,SIGTERM); waitpid(pid2,NULL,NULL);
 											kill(pid3,SIGTERM); waitpid(pid3,NULL,NULL);
 											kill(pid4,SIGTERM); waitpid(pid4,NULL,NULL);
@@ -856,10 +877,26 @@ int main(int argc, char *argv[])
 					}
 				} else if (datacheck < 0) {
 #ifdef DEBUG
-					DebugMsg("Select error, sleeping...");
+					DebugMsg("Select error, restarting agent");
 #endif //DEBUG
-					sleep(1);
-					continue;
+					// This error could mean that praudit has died or gone away
+					if (caught_usr1 || caught_pipe || caught_kill) {
+						DebugMsg("Attempting to restart\n");
+						close(praudit2daemon[0]);
+						//caught_kill=1;
+						//start killing pids
+						restart_agent = 1;
+						kill(pid2,SIGTERM); waitpid(pid2,NULL,NULL);
+						kill(pid3,SIGTERM); waitpid(pid3,NULL,NULL);
+						kill(pid4,SIGTERM); waitpid(pid4,NULL,NULL);
+						kill(ppid,SIGPIPE);
+						if (remote_allow) {
+							DebugMsg("Terminating web server...");
+						}
+						exit(0);
+					} else {
+						continue;
+					}
 				}
 
 				bytesread=read(praudit2daemon[0],bufferptr,spaceavail);
@@ -904,6 +941,7 @@ int main(int argc, char *argv[])
 							if (EvtCurrent) {
 								time_t currenttime;
 								struct tm *newtime;
+								char *psterm;
 
 								time(&currenttime);
 								newtime = localtime(&currenttime);
@@ -915,6 +953,13 @@ int main(int argc, char *argv[])
 								EvtCurrent->returncode = returncode;
 								strncpy(EvtCurrent->username, username, sizeof(username));
 								strncpy(EvtCurrent->searchterm, searchterm, sizeof(searchterm));
+								psterm = EvtCurrent->searchterm;
+								while (*psterm) {
+									if (*psterm == '%' || *psterm == '<' || *psterm == '>' || *psterm == '&') {
+										*psterm = '_';
+									}
+									psterm++;
+								}
 								strncpy(EvtCurrent->eventname, eventname, sizeof(eventname));
 								strncpy(EvtCurrent->options, options, sizeof(options));
 								EvtCurrent->seenflag=0;
@@ -1312,6 +1357,10 @@ int read_config_file(int nosetaudit)
 							SyslogDestination = 13;
 							DebugMsg("WARNING: Could not establish the correct syslog destination from the audit configuration: %s - Sending to user.notice\n", inputbuffer);
 						}
+					} else if (regmatchi(inputbuffer, "^restart=")) {
+						if (regmatchi(inputbuffer, "=0")) {
+							restart_allow = 0;
+						}
 					}
 				} else if (headertype == CONFIG_HOSTID) {
 					gethostident(inputbuffer, hostid, MAX_HOSTID);
@@ -1525,6 +1574,7 @@ int open_audit_network(char *string)
 			AuditDestination |= AUDIT_TO_STDOUT;
 			return(0);
 		}
+		return(1);
 	}
 
 	AuditDestination |= AUDIT_TO_STDOUT;
@@ -1784,7 +1834,7 @@ int splitobjective(char *string, char *event, char *user, char *match, int *excl
 	if (startevent == (char *)NULL || startuser == (char *)NULL ||
 	   startmatch == (char *)NULL || startcrit == (char *)NULL || startreturn == (char *)NULL) {
 		// Problem, this line is malformed. We really cannot proceed with this line.
-		DebugMsg("The following line does not contain the criticality, event, return, user and match elements: %s", string);
+		DebugMsg("The following line does not contain the criticality, event, return, user and match elements: [%s]", string);
 		return(-1);
 	}
 
@@ -1933,7 +1983,7 @@ Node *AddToList(char *eventname, char *username, char *path, int criticality, ch
 		// Find real eventname..
 		eventname2comment(eventname,comment,sizeof(comment));
 	} else {
-	strncpy(comment,"AUDIT_ALL",sizeof(comment));
+		strncpy(comment,"AUDIT_ALL",sizeof(comment));
 	}
 
 	if (newNode == NULL) {
@@ -2206,8 +2256,8 @@ int fix_last_error(HostNode *myhostcurrent) {
 	time_t now;
 	time(&now);
 	if (now - myhostcurrent->last_error >= WAITTIME) {
-		// try to reestablish the connection
-		if (myhostcurrent->AuditSocket) close(myhostcurrent->AuditSocket);
+		//try to reestablish the connection
+		if(myhostcurrent->AuditSocket) close(myhostcurrent->AuditSocket);
 		myhostcurrent->AuditSocket=0;
 		myhostcurrent->AuditSocket = socket(AF_INET,SOCK_DGRAM,0);
 		if (myhostcurrent->AuditSocket < 0) {
@@ -3128,7 +3178,7 @@ void start_web () {
 	while (!caught_kill && !caught_pipe) {
 		ret = ListenThread(&lasttime);
 
-		if (!ret) break;
+		//if (!ret) break;
 	}
 	CloseWebServer();
 	//while((ret = StartThread()) == 1) {
